@@ -4,7 +4,7 @@ try:
 except ImportError:
     import simplejson as json
 
-from rdflib.py3compat import PY3, format_doctest_out
+from ._compat import IS_PY3 as PY3
 
 from os import sep
 from os.path import normpath
@@ -41,7 +41,6 @@ def split_iri(iri):
             return iri[:at+1], iri[at+1:]
     return iri, None
 
-@format_doctest_out
 def norm_url(base, url):
     """
     >>> norm_url('http://example.org/', '/one')
@@ -54,6 +53,8 @@ def norm_url(base, url):
     'http://example.org/one/two'
     >>> norm_url('http://example.org/', 'http://example.net/one')
     'http://example.net/one'
+    >>> norm_url('http://example.org/', 'http://example.org//one')
+    'http://example.org//one'
     """
     parts = urlsplit(urljoin(base, url))
     path = normpath(parts[2])
